--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--
--*******************************************************************************
--
--  File name :       deskew.v
--
--  Description :     This module implements the training control
-- module. The training module implements the algorithm
-- for word alignment and bit alignment 
--                    
--  Date - revision : 11/30/2004
--
--  Author :          Tze Yi Yeoh
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************


library IEEE;
use IEEE.std_logic_1164.all;
--use IEEE.Std_logic_arith.all;
use IEEE.Std_logic_unsigned.all;
use IEEE.numeric_std.all;
--use IEEE.numeric_bit.all;
use work.all;

library unisim;
use unisim.vcomponents.all;

ENTITY deskew IS
   PORT (
      datain                  : IN std_logic_vector(3 DOWNTO 0);   
      edgei                   : OUT std_logic_vector(2 DOWNTO 0);   
      rst                     : IN std_logic;   
      deskew_en               : IN std_logic;   
      clkdiv                  : IN std_logic;   
      dlyce                   : OUT std_logic;   
      dlyinc                  : OUT std_logic;   
      done                    : OUT std_logic);   
END deskew;

ARCHITECTURE deskew_arch OF deskew IS

   SIGNAL counter                  :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL center_store             :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL edgei_init               :  std_logic_vector(2 DOWNTO 0);   
   SIGNAL cnt_rst                  :  std_logic;   
   SIGNAL cnt_inc                  :  std_logic;   
   SIGNAL loadedgei                :  std_logic;   
   SIGNAL load_center              :  std_logic;   
   SIGNAL center                   :  std_logic_vector(7 DOWNTO 0); 
    
   --Declare state machine parameters
   -- Deskew Control State Machine
   CONSTANT  START                 :  std_logic_vector(4 DOWNTO 0) := "00000"; -- 0    
   CONSTANT  START_WAIT1           :  std_logic_vector(4 DOWNTO 0) := "00001"; -- 1   
   CONSTANT  START_WAIT2           :  std_logic_vector(4 DOWNTO 0) := "00010"; -- 2  
   CONSTANT  SEEK_EDGE             :  std_logic_vector(4 DOWNTO 0) := "00011"; -- 3   
   CONSTANT  INC1                   :  std_logic_vector(4 DOWNTO 0) := "00100"; -- 4   
   CONSTANT  INC1_WAIT1             :  std_logic_vector(4 DOWNTO 0) := "00101"; -- 5   
   CONSTANT  INC1_WAIT2             :  std_logic_vector(4 DOWNTO 0) := "00110"; -- 6   
   CONSTANT  INC1_WAIT3             :  std_logic_vector(4 DOWNTO 0) := "00111"; -- 7   
   CONSTANT  EDGE1                 :  std_logic_vector(4 DOWNTO 0) := "01000"; -- 8   
   CONSTANT  INC2                   :  std_logic_vector(4 DOWNTO 0) := "01001"; -- 9   
   CONSTANT  INC2_WAIT1             :  std_logic_vector(4 DOWNTO 0) := "01010"; -- 10   
   CONSTANT  INC2_WAIT2             :  std_logic_vector(4 DOWNTO 0) := "01011"; -- 11   
   CONSTANT  INC2_WAIT3             :  std_logic_vector(4 DOWNTO 0) := "01100"; -- 12   
   CONSTANT  EDGE2                 :  std_logic_vector(4 DOWNTO 0) := "01101"; -- 13   
   CONSTANT  EDGE2_WAIT1           :  std_logic_vector(4 DOWNTO 0) := "01110"; -- 14   
   CONSTANT  CENTER_DEC            :  std_logic_vector(4 DOWNTO 0) := "01111"; -- 15   
   CONSTANT  CHECK_CENTER          :  std_logic_vector(4 DOWNTO 0) := "10000"; -- 16   
   CONSTANT  DESKEW_DONE           :  std_logic_vector(4 DOWNTO 0) := "10001"; -- 17   
   CONSTANT  DONE_WAIT1            :  std_logic_vector(4 DOWNTO 0) := "10010"; -- 18   
   CONSTANT  DONE_WAIT2            :  std_logic_vector(4 DOWNTO 0) := "10011"; -- 19   

   SIGNAL Current_State            :  std_logic_vector(4 DOWNTO 0):= START	;   
   SIGNAL Next_State               :  std_logic_vector(4 DOWNTO 0);   

   SIGNAL edgei_int              :  std_logic_vector(2 DOWNTO 0);   
   SIGNAL edgei_int_red_or       :  std_logic; 
   SIGNAL dlyce_int              :  std_logic;   
   SIGNAL dlyinc_int             :  std_logic;   
   SIGNAL done_int               :  std_logic;   

BEGIN
   edgei <= edgei_int;
   dlyce <= dlyce_int;
   dlyinc <= dlyinc_int;
   done <= done_int;

   -- Divide tap_counter by 2 to get mid point 
   center <= STD_LOGIC_VECTOR(UNSIGNED(center_store) SRL 1);

   -- Create edge information (Synthesis)
   edgei_int(2) <= datain(3) XOR datain(2) ;
   edgei_int(1) <= datain(2) XOR datain(1) ;
   edgei_int(0) <= datain(1) XOR datain(0) ;

   -- Reduction OR edgei_int signal
   edgei_int_red_or <= edgei_int(2) OR edgei_int(1) OR edgei_int(0);  

   -- All-purpose counter
   PROCESS (clkdiv, rst)
   BEGIN
      IF (rst = '1') THEN
         counter <= (others=>'0');   
      ELSIF rising_edge(clkdiv) THEN
         IF (cnt_rst = '1') THEN
            counter <= (others=>'0');    
         ELSIF (cnt_inc = '1') THEN
            counter <= counter + X"01";    
         END IF;
      END IF;
   END PROCESS;

   -- Load center count
   PROCESS (clkdiv, rst)
   BEGIN
      IF (rst = '1') THEN
         center_store <= (others=>'0');    
      ELSIF rising_edge(clkdiv) THEN
         IF (load_center = '1') THEN
            center_store <= counter;    
         END IF;
      END IF;
   END PROCESS;

   -- Load initial edge information
   PROCESS (clkdiv, rst)
   BEGIN
      IF (rst = '1') THEN
         edgei_init <= (others=>'0');    
      ELSIF rising_edge(clkdiv) THEN
         IF (loadedgei = '1') THEN
            edgei_init <= edgei_int;    
         END IF;
      END IF;
   END PROCESS;

   -- Current State Logic
   PROCESS (clkdiv, rst)
   BEGIN
      IF (rst = '1') THEN
         Current_State <= START;    
      ELSIF rising_edge(clkdiv) THEN
		   IF (deskew_en = '1') THEN
            Current_State <= Next_State;    
			END IF;
      END IF;
   END PROCESS;

   -- Output forming logic
   PROCESS (Current_State)
   BEGIN
      cnt_rst <= '0';    
      cnt_inc <= '0';    
      dlyce_int <= '0';    
      dlyinc_int <= '0';    
      loadedgei <= '0';    
      load_center <= '0';    
      done_int <= '0';   

      CASE Current_State IS
         WHEN START =>
                  cnt_rst <= '1';     
			    
	    -- Look for a pattern out of the ISERDES that contains an edge
	    -- which can be monitored for movement. 			     
         WHEN SEEK_EDGE =>   
                  loadedgei <= '1';    

	    -- Increment tap delay line until next left edge is found (edge pattern
	    -- shifts right by one)			      
         WHEN INC1 =>
                  dlyce_int <= '1';    
                  dlyinc_int <= '1';    

	    -- If the first edge has been found, continue incrememnting 
	    -- the tap delay line while incrementing the tap counter
	    -- until the second edge is found. 			     
         WHEN INC2 =>  
                  cnt_inc <= '1';    
                  dlyce_int <= '1';    
                  dlyinc_int <= '1';    
			      			      
         WHEN EDGE2_WAIT1 =>
                  cnt_rst <= '1';    
                  load_center <= '1';    

	    -- Once second edge is found, decrement the tap delay line to half
	    -- the value of the tap counter 			     
         WHEN CENTER_DEC =>
                  cnt_inc <= '1';    
                  dlyce_int <= '1';    
			     
         WHEN DESKEW_DONE =>
                  done_int <= '1'; 

         WHEN OTHERS  =>
                  null;
									      
      END CASE;
   END PROCESS;

   PROCESS (Current_State, counter, edgei_int, edgei_init, center)
   BEGIN
      CASE Current_State IS
         WHEN START =>
                  Next_State <= START_WAIT1;    

	    -- Insert wait states after START to account for pipeline stages
	    -- in channel select MUX.
         WHEN START_WAIT1 =>
                  Next_State <= START_WAIT2;  
			     
         WHEN START_WAIT2 =>
                  Next_State <= SEEK_EDGE; 
			      
         WHEN SEEK_EDGE =>
                  IF ((edgei_int(2) OR edgei_int(1)) = '1') THEN
                     Next_State <= INC1;  				   
                  ELSE
                     Next_State <= SEEK_EDGE;    
                  END IF;

	    -- Increment tap delay line until next left edge is found (edge pattern
	    -- shifts right by one)
         WHEN INC1 =>
                  Next_State <= INC1_WAIT1; 
			      
         WHEN INC1_WAIT1 =>
                  Next_State <= INC1_WAIT2;  	  
		 

         WHEN INC1_WAIT2 =>
                  Next_State <= INC1_WAIT3; 
	   			      
         WHEN INC1_WAIT3 =>
                  Next_State <= EDGE1;   

	    -- Check to see if the Q1 output of the ISERDES has changed 
	    -- indicating the first edge has been found.			    
         WHEN EDGE1 =>
                  IF (edgei_int = STD_LOGIC_VECTOR(UNSIGNED(edgei_init) SRL 1)) THEN
                     Next_State <= INC2;    
                  ELSE
                     Next_State <= INC1;    
                  END IF;

         WHEN INC2 =>
                  Next_State <= INC2_WAIT1; 
			      
         WHEN INC2_WAIT1 =>
                  Next_State <= INC2_WAIT2;
			       
         WHEN INC2_WAIT2 =>
                  Next_State <= INC2_WAIT3;
			       
         WHEN INC2_WAIT3 =>
                  Next_State <= EDGE2; 

	    -- Check to see if the Q2 output of the ISERDES has changed 
	    -- indicating the second edge has been found.			      
         WHEN EDGE2 =>
                  IF (edgei_int = STD_LOGIC_VECTOR(UNSIGNED(edgei_init) SRL 2)) THEN
                     Next_State <= EDGE2_WAIT1;    
                  ELSE
                     Next_State <= INC2;    
                  END IF;

         WHEN EDGE2_WAIT1 =>
                  Next_State <= CENTER_DEC; 

	    -- Once second edge is found, decrement the tap delay line to half
	    -- the value of the tap counter 			      
         WHEN CENTER_DEC =>
                  Next_State <= CHECK_CENTER;  
			     
         WHEN CHECK_CENTER =>
                  IF (counter = center) THEN
                     Next_State <= DESKEW_DONE;    
                  ELSE
                     Next_State <= CENTER_DEC;    
                  END IF;

	    -- Check to see if the center point of the data eye has been reached.
         WHEN DESKEW_DONE =>
                  Next_State <= DONE_WAIT1; 
			      
         WHEN DONE_WAIT1 =>
                  Next_State <= DONE_WAIT2; 
			      
         WHEN DONE_WAIT2 =>
                  Next_State <= START;  
			     
         WHEN OTHERS  =>
                  Next_State <= START;    
         
      END CASE;
   END PROCESS;

END deskew_arch;
