--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--
--*******************************************************************************
--
--  File name :       monitor_ctrl.vhd
--
--  Description :     This module implements the real time window 
--                    monitoring control module.  
--                    
--  Date - revision : 11/30/2004
--
--  Author :          Tze Yi Yeoh
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.Std_logic_arith.all;
use IEEE.Std_logic_unsigned.all;
use IEEE.numeric_std.all;
use IEEE.numeric_bit.all;
use work.all;

library unisim;
use unisim.vcomponents.all;

ENTITY monitor_ctrl IS
   PORT (
      datain                  : IN std_logic_vector(3 DOWNTO 0);   
      datain_monitor          : IN std_logic_vector(3 DOWNTO 0);   
      rst                     : IN std_logic;   
      monitor_en              : IN std_logic;   
      clkdiv                  : IN std_logic;   
      dlyce_window            : OUT std_logic;   
      dlyinc_window           : OUT std_logic;   
      dlyce                   : OUT std_logic;   
      dlyinc                  : OUT std_logic;   
      monitor_done            : OUT std_logic);   
END ENTITY monitor_ctrl;

ARCHITECTURE monitor_ctrl_arch OF monitor_ctrl IS 
   SIGNAL flag_right               :  std_logic;   
   SIGNAL flag_left                :  std_logic;   
   SIGNAL counter                  :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL cnt_rst                  :  std_logic;   
   SIGNAL cnt_inc                  :  std_logic;   

   -- In this example, width of window = 6 taps (500 ps) for 1Gbps
   -- Set the WIDTH of the monitoring window.
   CONSTANT  WIDTH                 :  std_logic_vector(6 DOWNTO 0) := "0000110";    

   --Declare state machine parameters
   -- Window Monitoring Control State Machine
   CONSTANT  START                 :  std_logic_vector(4 DOWNTO 0) := "00000";    -- 0
   CONSTANT  START_WAIT1           :  std_logic_vector(4 DOWNTO 0) := "00001";    -- 1
   CONSTANT  START_WAIT2           :  std_logic_vector(4 DOWNTO 0) := "00010";    -- 2
   CONSTANT  DEC                   :  std_logic_vector(4 DOWNTO 0) := "00011";    -- 3
   CONSTANT  WAIT_DEC              :  std_logic_vector(4 DOWNTO 0) := "00100";    -- 4
   CONSTANT  CHECK_LEFT            :  std_logic_vector(4 DOWNTO 0) := "00101";    -- 5
   CONSTANT  LEFT_EDGE             :  std_logic_vector(4 DOWNTO 0) := "00110";    -- 6
   CONSTANT  INC                   :  std_logic_vector(4 DOWNTO 0) := "00111";    -- 7
   CONSTANT  WAIT_INC              :  std_logic_vector(4 DOWNTO 0) := "01000";    -- 8
   CONSTANT  CHECK_RIGHT           :  std_logic_vector(4 DOWNTO 0) := "01001";    -- 9
   CONSTANT  RIGHT_EDGE            :  std_logic_vector(4 DOWNTO 0) := "01010";    -- 10
   CONSTANT  MOVE_RIGHT            :  std_logic_vector(4 DOWNTO 0) := "01011";    -- 11
   CONSTANT  WAIT_MOVE_RIGHT1      :  std_logic_vector(4 DOWNTO 0) := "01100";    -- 12
   CONSTANT  WAIT_MOVE_RIGHT2      :  std_logic_vector(4 DOWNTO 0) := "01101";    -- 13
   CONSTANT  MOVE_LEFT             :  std_logic_vector(4 DOWNTO 0) := "01110";    -- 14
   CONSTANT  WAIT_MOVE_LEFT1       :  std_logic_vector(4 DOWNTO 0) := "01111";    -- 15
   CONSTANT  WAIT_MOVE_LEFT2       :  std_logic_vector(4 DOWNTO 0) := "10000";    -- 16
   CONSTANT  DONE                  :  std_logic_vector(4 DOWNTO 0) := "10001";    -- 17

   SIGNAL Current_State            :  std_logic_vector(4 DOWNTO 0) := START;   
   SIGNAL Next_State               :  std_logic_vector(4 DOWNTO 0);  

   SIGNAL dlyce_window_int       :  std_logic;   
   SIGNAL dlyinc_window_int      :  std_logic;   
   SIGNAL dlyce_int              :  std_logic;   
   SIGNAL dlyinc_int             :  std_logic;   
   SIGNAL monitor_done_int       :  std_logic;   

BEGIN
   dlyce_window <= dlyce_window_int;
   dlyinc_window <= dlyinc_window_int;
   dlyce <= dlyce_int;
   dlyinc <= dlyinc_int;
   monitor_done <= monitor_done_int;

   -- All-purpose counter
   PROCESS (clkdiv, rst)
   BEGIN
      IF (rst = '1') THEN
         counter <= (others=>'0');   
      ELSIF rising_edge(clkdiv) THEN
         IF (cnt_rst = '1') THEN
            counter <= (others=>'0');    
         ELSIF (cnt_inc = '1') THEN
            counter <= counter + X"01";
         END IF;
      END IF;
   END PROCESS;

   -- Current State Logic
   PROCESS (clkdiv, rst)
   BEGIN
      IF (rst = '1') THEN
         Current_State <= START; 			   
      ELSIF rising_edge(clkdiv) THEN
		   IF (monitor_en = '1') THEN
            Current_State <= Next_State;    
			END IF;
      END IF;
   END PROCESS;

   -- Output forming logic
   PROCESS (Current_State, datain, datain_monitor)
   BEGIN
      dlyce_window_int <= '0';    
      dlyinc_window_int <= '0';    
      dlyce_int <= '0';    
      dlyinc_int <= '0';    
      cnt_rst <= '0';    
      cnt_inc <= '0';    
      flag_right <= '0';    
      flag_left <= '0';    
      monitor_done_int <= '0';    

      CASE Current_State IS
         WHEN START =>   
                  cnt_rst <= '1';    

         -- Decrement tap delay chain by 1 tap         
         WHEN DEC =>
                  dlyce_window_int <= '1';    
                  cnt_inc <= '1';    
			    
         -- Check to see if left edge of monitoring window reached         
         WHEN LEFT_EDGE =>			    
                     cnt_rst <= '1';    
                     -- If the left edge of the monitoring window is reached,                      
                     -- compare datain from Data ISERDES and datain from                      
                     -- Monitor ISERDES. If datain Data ISERDES does not match datain                      
                     -- Monitor ISERDES, increment tap delay chain for both Data ISERDES                     
                     -- and Monitor ISERDES.                     
                     IF (datain /= NOT datain_monitor) THEN
                        -- datain_monitor = ~datain due to IBUFDS_DIFF_OUT                        
                        flag_left <= '1';    
		               END IF;	
			   	   
         -- Increment tap delay chain by 1 tap         
         WHEN INC =>
                  dlyce_window_int <= '1';    
                  dlyinc_window_int <= '1';      
                  cnt_inc <= '1';    
   			    
         -- Check to see if right edge of monitoring window reached         
         WHEN RIGHT_EDGE =>
                     cnt_rst <= '1';    
                     -- If the right edge of the monitoring window is reached,                      
                     -- compare datain from Data ISERDES and datain from                      
                     -- Monitor ISERDES. If datain Data ISERDES does not match datain                      
                     -- Monitor ISERDES, decrement tap delay chain for both Data ISERDES                     
                     -- and Monitor ISERDES.                     
                     IF (datain /= NOT datain_monitor) THEN
                        -- datain_monitor = ~datain due to IBUFDS_DIFF_OUT                        
                        flag_right <= '1';    
                     END IF;			      

         -- Increment the tap delay chain for both the Data ISERDES and          
         -- Monitor ISERDES when data falls outside the left edge of the          
         -- data valid window.         
         WHEN MOVE_RIGHT =>
                  dlyce_window_int <= '1';    
                  dlyinc_window_int <= '1';    
                  dlyce_int <= '1';    
                  dlyinc_int <= '1';    

         -- Decrement the tap delay chain for both the Data ISERDES and          
         -- Monitor ISERDES when data falls outside the right edge of the          
         -- data valid window.         
         WHEN MOVE_LEFT =>
                  dlyce_window_int <= '1';     
                  dlyce_int <= '1';    

         WHEN DONE =>
                  monitor_done_int <= '1';   

         WHEN OTHERS  =>
                  null;
												          
      END CASE;
   END PROCESS;

   PROCESS (Current_State, counter, datain, datain_monitor)
   BEGIN
      CASE Current_State IS
         WHEN START =>
                     Next_State <= START_WAIT1;    

         WHEN START_WAIT1 =>
                  Next_State <= START_WAIT2;    

         WHEN START_WAIT2 =>
                  Next_State <= DEC;  
						  
         -- Decrement tap delay chain by 1 tap         
         WHEN DEC =>
                  Next_State <= WAIT_DEC;   
						 
         WHEN WAIT_DEC =>
                  Next_State <= CHECK_LEFT;  
						  
         -- Check to see if left edge of monitoring window reached         
         WHEN CHECK_LEFT =>
                  -- If left edge of the monitoring window reached                  
                  IF (counter = WIDTH) THEN
                     -- If the left edge of the monitoring window is reached,                      
                     -- compare datain from Data ISERDES and datain from                      
                     -- Monitor ISERDES. If datain Data ISERDES does not match datain                      
                     -- Monitor ISERDES, increment tap delay chain for both Data ISERDES                    
                     -- and Monitor ISERDES.                                          
                     Next_State <= LEFT_EDGE;                     
                  ELSE
                     Next_State <= DEC;    
                  END IF;

         WHEN LEFT_EDGE =>
                  -- If left edge of the monitoring window reached                  
                  IF (datain = NOT datain_monitor) THEN
                     -- If the left edge of the monitoring window is reached,                      
                     -- compare datain from Data ISERDES and datain from                      
                     -- Monitor ISERDES. If datain Data ISERDES does not match datain                      
                     -- Monitor ISERDES, increment tap delay chain for both Data ISERDES                    
                     -- and Monitor ISERDES.                                           
                     Next_State <= INC;                     
                  ELSE
                     Next_State <= MOVE_RIGHT;    
                  END IF;

         -- Increment tap delay chain by 1 tap         
         WHEN INC =>
                  Next_State <= WAIT_INC;  
						  
         WHEN WAIT_INC =>
                  Next_State <= CHECK_RIGHT;  
			     
         -- Check to see if right edge of monitoring window reached         
         WHEN CHECK_RIGHT =>
                  -- If left edge of the monitoring window reached                  
                  IF (counter = WIDTH) THEN
                     -- If the right edge of the monitoring window is reached,                      
                     -- compare datain from Data ISERDES and datain from                      
                     -- Monitor ISERDES. If datain Data ISERDES does not match datain                      
                     -- Monitor ISERDES, decrement tap delay chain for both Data ISERDES                     
                     -- and Monitor ISERDES.                     
                     Next_State <= RIGHT_EDGE;    
                  ELSE
                     -- If right edge of the monitoring window not reached, keep                     
                     -- incrementing tap delay chain of Monitor ISERDES.		                        
                     Next_State <= INC;    
                  END IF;

         WHEN RIGHT_EDGE =>
                  IF (datain = NOT datain_monitor) THEN
                     -- If the right edge of the monitoring window is reached,                      
                     -- compare datain from Data ISERDES and datain from                      
                     -- Monitor ISERDES. If datain Data ISERDES does not match datain                      
                     -- Monitor ISERDES, decrement tap delay chain for both Data ISERDES                     
                     -- and Monitor ISERDES.                     
                     Next_State <= DONE;    
                  ELSE
                     -- If right edge of the monitoring window not reached, keep                     
                     -- incrementing tap delay chain of Monitor ISERDES.		                        
                     Next_State <= MOVE_LEFT;    
                  END IF;

         -- Increment the tap delay chain for both the Data ISERDES and          
         -- Monitor ISERDES when data falls outside the left edge of the          
         -- data valid window.         
         WHEN MOVE_RIGHT =>
                  Next_State <= WAIT_MOVE_RIGHT1; 
						   
         WHEN WAIT_MOVE_RIGHT1 =>
                  Next_State <= WAIT_MOVE_RIGHT2;   

         WHEN WAIT_MOVE_RIGHT2 =>
                  Next_State <= LEFT_EDGE;    
												 
         -- Decrement the tap delay chain for both the Data ISERDES and          
         -- Monitor ISERDES when data falls outside the right edge of the          
         -- data valid window.         
         WHEN MOVE_LEFT =>
                  Next_State <= WAIT_MOVE_LEFT1; 
						   
         WHEN WAIT_MOVE_LEFT1 =>
                  Next_State <= WAIT_MOVE_LEFT2;  

         WHEN WAIT_MOVE_LEFT2 =>
                  Next_State <= RIGHT_EDGE;   
												  
         WHEN DONE =>
                  Next_State <= START; 
						   
         WHEN OTHERS  =>
                  Next_State <= START;             
      END CASE;
   END PROCESS;

END ARCHITECTURE monitor_ctrl_arch;
